#!/bin/ash
#Been trying to use puppy approach to install/uninstall packages. Think that starting from scratch (pUPnGO) is different from using normal puppy when it comes to package install: For pUPnGO almost everything will be an "alien" package - and for the packager knowing what to include or not will be hard to guess. So one way could be to request all packages to include whatever needed to be able to run after install in the "pristine" pUPnGO. This definitely moves the overhead from the OS to the packages - but pUPnGO was meant to be used for "embedded" or single application systems. If one wants to build a feature rich desktop OS you would probably not start from the level where pUPnGO is. 
#My approach for now is to have one database for installed packages and their files with the simple structure of: "package-file-name" "filepath". At install database is read, if package is present abort else unpack, write to database, and install file. At uninstall database is read, determine if file to remove is shared with other packages: If shared delete package entry but not file, if not shared check if file is a "core"-file (from pUPnGO-main.sfs), delete file and if core copy the core file.
#I have tested this approach in my running daily puppy412 and it seems to work. Error-checking might be improved though.
#Attached very simple scripts for install and uninstall - no-GUI yet - to demonstrate the principle. I will appreciate any feed-back on the principle as I might not be aware of every aspect of installing and uninstalling in puppy.
#To maintain compatability with org. P412 creation of packagename.files and alienpackage.txt is maintained (but not used by the scripts).
#updated for pupngo041210: install SFS, tgz, rpm, deb
#updated 080211: added tcz and silent run for comp. with tc_412.sfs
#updated 280211: added tar.gz install
#revised 300412 - ash/bash syntax
clear

if [ $(which dialogfunctions.sh) ]; then #check functions present
INCLUDE=$(which dialogfunctions.sh)
else
echo "You are missing file 'dialogfunctions.sh'.
Program will not run without it."
exit
fi

CURDIR=$(pwd)
. "${INCLUDE}"	#contains functions used for the dialogs

TITLE="Package Installer"

[ ! -d /root/.packages ] && mkdir /root/.packages
[ ! -d /tmp/pet ] && mkdir /tmp/pet
#[ ! -f /tmp/PUPNGOERRORS.log ] && echo "" > /tmp/PUPNGOERRORS.log
exec 2>/tmp/PUPNGOERRORS.log
[ ! -f /root/.packages/alienpackages.txt ] && touch /root/.packages/alienpackages.txt
[ ! -f /root/.packages/allfiles.txt ] && touch /root/.packages/allfiles.txt

COMPATIBLE=""	#set to "yes" if you want org P412 compatability
DATABASE="/root/.packages/allfiles.txt"

[ $(ps | grep tce-load) ] && SILENT="yes"	#for tc_412.sfs


check_input () {
[ "${PACKAGENAME}" = "reset" ] && PACKAGENAME=""	#quick hack to avoid call with reset from setup.sh
#echo "debug: in check_input" >> /tmp/debug.txt
if [ ! "${SILENT}" = "yes" ]; then
  if [ "${PACKAGENAME}" = "" ]; then
    TEXT="Please enter full path to pet-file below. Current
directory is: $CURDIR.
"
    # input_fc
    fileselect_fc

    # evaluate response
    if [ "$REPLY_NUM" = "cancel" ]; then
	  echo "Canceled"
	  exit
    else
      PACKAGENAME="$(cat /tmp/reply)"
    fi
  fi
else
  if [ "${PACKAGENAME}" = "" ]; then
    exit		#prevents silent run from installing with empty filename
  fi
fi


if [ ! -f "${PACKAGENAME}" ]; then
TEXT="The package-file - ${PACKAGENAME} - could not be found.
"
message_fc
exit
fi
}

check_file_type () {
#PACKAGETYPE="${PACKAGENAME/*./}"	#fails with ash - works with bash
#PACKAGETYPE="${PACKAGENAME##*.}"	#works with both
PACKAGETYPE="$(echo $PACKAGENAME | awk -F . '{print $NF}')"


#echo "debug: PACKAGETYPE $PACKAGETYPE" >> /tmp/debug.txt

case "${PACKAGETYPE}" in
pet) TEXT="";;
sfs) TEXT="";;
tcz) TEXT="";;
tgz) TEXT="";;
rpm) TEXT="";;
deb) TEXT="";;
gz)  #echo "debug in gz): PACKAGETYPE $PACKAGETYPE" >> /tmp/debug.txt
     if [ ! "$(echo $PACKAGENAME | grep .tar.gz)" = "" ]; then
       TEXT=""
     else
       TEXT="Seems not to be a package that can be installed."
     fi;;
*)   TEXT="Seems not to be a package that can be installed."
esac
[ ! "${TEXT}" = "" ] && message_fc && exit
}

cleanup () {
rm -rf /tmp/pet/"${NEWNAME}"
rm -f /tmp/pet/"${NEWNAME}".list
rmdir /tmp/pet
rm -f /tmp/select
rm -f /tmp/choices
rm -f /tmp/reply
rm -f /tmp/dlg
}


list_files () {
cd "${1}"
#find * > /root/.packages/"${PACKAGENAME/.pet/}".files.temp	#not ash - only bash
#${F%.*}

find * > /root/.packages/"${PACKAGENAME%.*}".pet.files.temp
#above file holds all files in package
sync

while read ONELINE; do
echo "\"$PACKAGENAME\" \"/$ONELINE\"" >> "${DATABASE}"
#create file for compatability with org. P412 installer
if [ "${COMPATIBLE}" = "yes" ]; then
echo "/$ONELINE" >> /root/.packages/"${PACKAGENAME%.*}".pet.files
fi
done < /root/.packages/"${PACKAGENAME%.*}".pet.files.temp

rm -f /root/.packages/"${PACKAGENAME%.*}".pet.files.temp
sort "${DATABASE}" > "${DATABASE}".tmp
mv -f "${DATABASE}".tmp "${DATABASE}"
}

install () {
PACKAGENAME=$(basename ${PACKAGENAME})
#is package already installed?
COUNT=$(grep -c "^\"${PACKAGENAME}\"" "${DATABASE}")
if [ ! "${COUNT}" = "0" ]; then
TEXT="A package with the name - ${PACKAGENAME} - is already installed.
You will have to remove the package before doing a re-install.
Would you like to run the uninstaller now?"

#write CHOICES to file
echo "No
Yes" >/tmp/choices
#run...
menu_fc     
   
#evaluate response
menuitem=$(cat /tmp/reply) 2> /dev/null
case $menuitem in
  0) cleanup
     exit;;
  1) pkgremove.sh;;
esac

cleanup
exit
fi
list_files /tmp/pet/"${NEWNAME}"/	#function call

cd "${CURDIR}"
cp -af  /tmp/pet/"${NEWNAME}"/* /

if [ "${PACKAGETYPE}" = "sfs" ]; then
  TESTBUG=$(df | grep /tmp/pet/$NEWNAME | cut -d " " -f 1)
  umount /tmp/pet/$NEWNAME
  sync
  losetup -d ${TESTBUG}
fi

if [ "${PACKAGETYPE}" = "tcz" ]; then
  TESTBUG=$(df | grep /tmp/pet/$NEWNAME | cut -d " " -f 1)
  umount /tmp/pet/$NEWNAME
  sync
  losetup -d ${TESTBUG}
fi

sync
rm -rf /tmp/pet/"${NEWNAME}"

ls /install/doinst.sh >/dev/null 2>/dev/null 
if [ $? = 0 ] ; then
  cd / 
  chmod +x /install/doinst.sh  
  /install/doinst.sh
  rm -r /install
  cd "${CURDIR}"
fi

if [ -s /pinstall.sh ]; then
/pinstall.sh
rm -f /pinstall.sh
fi

if [ -s /punstall.sh ]; then
mv -f /punstall.sh /root/.packages/"${NEWNAME}".remove
fi

[ -f /tmp/pet/"${NEWNAME}"/punstall.sh ] && mkdir /tmp/pet/"${NEWNAME}"/root/.packages
[ -f /tmp/pet/"${NEWNAME}"/punstall.sh ] && mv /tmp/pet/"${NEWNAME}"/punstall.sh /tmp/pet/"${NEWNAME}"/root/.packages/"${NEWNAME}".remove

#create file alienpackages.txt for compatability with org. puppy
if [ "${COMPATIBLE}" = "yes" ]; then
echo "\"${NEWNAME}\" \"${NEWNAME} : \" on \" \" \\" >> /root/.packages/alienpackages.txt
fi

ldconfig 2>/dev/null
sync
if [ ! "${SILENT}" = "yes" ]; then
  TEXT="Package-file ${PACKAGENAME} has been installed.
"
  message_fc
fi
}

unpack_pet () {
FILENAME=`basename "${PACKAGENAME}"`
NEWNAME="`echo $FILENAME | sed -e 's/.pet//g'`"
cp "${PACKAGENAME}" /tmp/pet/
cd /tmp/pet/
mv "${FILENAME}" "${NEWNAME}".gz
tar -xzf "${NEWNAME}".gz 2>/dev/null
rm -f /tmp/pet/"${NEWNAME}".gz
cd /tmp/pet/"${NEWNAME}"
rm -f /tmp/pet/"${NEWNAME}"/*.specs
rm -f /tmp/pet/"${NEWNAME}"/*.xpm
find * > /tmp/pet/"${NEWNAME}".list
sync
cd "${CURDIR}"
}

unpack_sfs () {
FILENAME=`basename "${PACKAGENAME}"`
NEWNAME="`echo $FILENAME | sed -e 's/.sfs//g'`"

imgFile="$PACKAGENAME"
[ -z "$imgFile" ] && TEXT="File does not exist." && message_fc && exit 1
if [ "$(dirname $imgFile)" = "\." ] ; then
  imgFile="$(pwd) $(echo $imgFile |sed 's/^\.//')"
fi	
	
MntPt="/mnt/"$(echo "$imgFile" |sed "s#^\.##g" |sed "s#/#+#g")
imgFileBASE="$(basename "$imgFile")"

mkdir -p /tmp/pet/$NEWNAME
mount -t squashfs -o loop $imgFile /tmp/pet/$NEWNAME
cd /tmp/pet/"${NEWNAME}"
find * > /tmp/pet/"${NEWNAME}".list
sync
cd "${CURDIR}"
}

chk_squash_version () {
#returns blank, 3 or 4
SQUASHFS_VERSION=$(disktype $1 | grep version | cut -f 4 -d " " | cut -c 1)
}

squashfs_conv_423 () {
THISAPP="$1"
unsquashfs4 "${THISAPP}" 2>&1 >/dev/null
rm -f "${THISAPP}"
mksquashfs3 squashfs-root "$1" 2>&1 >/dev/null
rm -r squashfs-root
}

unpack_tcz () {
#check for squashfs-type here
chk_squash_version "${PACKAGENAME}"
if [ "${SQUASHFS_VERSION}" = "" ]; then
  TEXT="$REPLY_CHAR seems not to be a supported squashfs-file."
  message_fc
  exit 1
fi

if [ $SQUASHFS_VERSION -eq 4 ]; then
  squashfs_conv_423 "${PACKAGENAME}"
fi

FILENAME=$(basename "${PACKAGENAME}")
NEWNAME="$(echo $FILENAME | sed -e 's/.tcz//g')"

imgFile="$PACKAGENAME"
[ -z "$imgFile" ] && TEXT="File does not exist." && message_fc && exit 1

if [ "$(dirname $imgFile)" = "\." ] ; then
  imgFile="$(pwd) $(echo $imgFile |sed 's/^\.//')"
fi	
	
MntPt="/mnt/"$(echo "$imgFile" |sed "s#^\.##g" |sed "s#/#+#g")
imgFileBASE="$(basename "$imgFile")"

mkdir -p /tmp/pet/$NEWNAME
mount -t squashfs -o loop $imgFile /tmp/pet/$NEWNAME
cd /tmp/pet/"${NEWNAME}"
find * > /tmp/pet/"${NEWNAME}".list
sync
cd "${CURDIR}"
}

unpack_tgz () {
FILENAME=$(basename "${PACKAGENAME}")
NEWNAME="$(echo $FILENAME | sed -e 's/.tgz//g')"
cp "${PACKAGENAME}" /tmp/pet/
cd /tmp/pet/
gunzip "${FILENAME}"	#gives tar-file
mkdir "${NEWNAME}"
cd /tmp/pet/"${NEWNAME}"
tar -xvf /tmp/pet/"${NEWNAME}".tar 2>&1 >/dev/null
rm -f /tmp/pet/"${NEWNAME}".tar
rm -f /tmp/pet/"${NEWNAME}"/*.specs
rm -f /tmp/pet/"${NEWNAME}"/*.xpm
find * > /tmp/pet/"${NEWNAME}".list
sync
cd "${CURDIR}"
}

unpack_rpm () {
FILENAME=$(basename "${PACKAGENAME}")
NEWNAME="$(echo $FILENAME | sed -e 's/.rpm//g')"
cd /tmp/pet/
mkdir "${NEWNAME}"
cd "${CURDIR}"
cp "${PACKAGENAME}" /tmp/pet/"${NEWNAME}"
cd /tmp/pet/"${NEWNAME}"
rpm2cpio $FILENAME | cpio -id
rm /tmp/pet/"${NEWNAME}"/"${PACKAGENAME}"
find * > /tmp/pet/"${NEWNAME}".list
sync
cd "${CURDIR}"
}

unpack_deb () {
FILENAME=$(basename "${PACKAGENAME}")
NEWNAME="$(echo $FILENAME | sed -e 's/.deb//g')"
cd /tmp/pet/
mkdir "${NEWNAME}"
cd "${CURDIR}"
cp "${PACKAGENAME}" /tmp/pet/"${NEWNAME}"
cd /tmp/pet/"${NEWNAME}"
ar p "${FILENAME}" data.tar.gz | tar zx
rm /tmp/pet/"${NEWNAME}"/"${PACKAGENAME}"
find * > /tmp/pet/"${NEWNAME}".list
sync
cd "${CURDIR}"
}

unpack_targz () {
FILENAME=$(basename "${PACKAGENAME}")
#echo "debug: in unpack_targx $FILENAME" >> /tmp/debug.txt

NEWNAME="$(echo $FILENAME | sed -e 's/.tar.gz//g')"

#echo "debug: in unpack_targx $NEWNAME" >> /tmp/debug.txt

mkdir -p /tmp/pet/"${NEWNAME}"

cp "${PACKAGENAME}" /tmp/pet/"${NEWNAME}"
cd /tmp/pet/"${NEWNAME}"

tar -zxf "${NEWNAME}".tar.gz 2>&1 >/dev/null
rm -f /tmp/pet/"${NEWNAME}"/"${NEWNAME}".tar.gz
rm -f /tmp/pet/"${NEWNAME}"/"${NEWNAME}"/*.specs
rm -f /tmp/pet/"${NEWNAME}"/"${NEWNAME}"/*.xpm
cd /tmp/pet/"${NEWNAME}"
find * > /tmp/pet/"${NEWNAME}".list
sync
cd "${CURDIR}"
}

#MAIN PROGRAM

PACKAGENAME=$1
#echo "debug: PACKAGENAME $PACKAGENAME" > /tmp/debug.txt


check_input 		#function call
check_file_type		#function call



[ "${PACKAGETYPE}" = "pet" ] && unpack_pet
[ "${PACKAGETYPE}" = "sfs" ] && unpack_sfs
[ "${PACKAGETYPE}" = "tcz" ] && unpack_tcz
[ "${PACKAGETYPE}" = "tgz" ] && unpack_tgz
[ "${PACKAGETYPE}" = "rpm" ] && unpack_rpm
[ "${PACKAGETYPE}" = "deb" ] && unpack_deb
[ "${PACKAGETYPE}" = "gz" ] && unpack_targz


[ ! "${SILENT}" = "yes" ] && echo "Please wait..."
install				#function call
cleanup


#error-check
[ -f /tmp/PUPNGOERRORS.log ] && ERRORS=$(cat /tmp/PUPNGOERRORS.log)

#echo "debug ERRORS $ERRORS" >>/tmp/debug.txt
if [ ! "${SILENT}" = "yes" ]; then
  if [ ! "${ERRORS}" = "" ]; then
  TEXT="The following messages or errors were encountered:
${ERRORS}
Errors from tar and gzip is normally harmless.
"
  message_fc
  fi
fi
exit