#!/bin/ash
#Special package remover for pUPnGO.
#updated for pupngo041210 (sfs, tgz, rpm, deb)
#updated 280211: remove tar.gz
#revised 300412 - ash/bash syntax
clear

if [ $(which dialogfunctions.sh) ]; then #check functions present
INCLUDE=$(which dialogfunctions.sh)
else
echo "You are missing file 'dialogfunctions.sh'.
Program will not run without it."
exit
fi

CURDIR=$(pwd)
. "${INCLUDE}"	#contains functions used for the dialogs

TITLE="Package Remover"

DATABASE="/root/.packages/allfiles.txt"

[ ! -f /tmp/PUPNGOERRORS.log ] && echo "" > /tmp/PUPNGOERRORS.log
exec 2>/tmp/PUPNGOERRORS.log


cleanup () {
rm -f /tmp/dirnames
rm -f /tmp/"${PETSEL}".files 
rm -f /tmp/validation
rm -f /tmp/select
rm -f /tmp/choices
rm -f /tmp/reply
rm -f /tmp/dlg
}

del_empty_dir () {

DIR="${1}"
while [[ ! $(find ${DIR} -type f) ]]; do


  if [ ! -d /initrd/pup_ro2"${DIR}" ]; then	#check if core-dir

   rmdir "${DIR}"
  fi

  DIR=$(dirname "${DIR}")
  sync
done	
}

#MAIN PROGRAM
#make list of user installed pets
cd /root/.packages/

#check if any pets installed yet
if [ "$(cat $DATABASE)" = "" ]; then
  TEXT="No packages found suitable for removal.
"
  message_fc
  cleanup   
  exit
else
  TEXT="Choose which package to remove:"
    
  #create choices
  echo "Cancel" > /tmp/choices
  grep ".pet\"" "${DATABASE}" | cut -d " " -f 1 | tr -d "\"" | sort -u >> /tmp/choices
  grep ".sfs\"" "${DATABASE}" | cut -d " " -f 1 | tr -d "\"" | sort -u >> /tmp/choices
  grep ".tcz\"" "${DATABASE}" | cut -d " " -f 1 | tr -d "\"" | sort -u >> /tmp/choices
  grep ".tgz\"" "${DATABASE}" | cut -d " " -f 1 | tr -d "\"" | sort -u >> /tmp/choices
  grep ".rpm\"" "${DATABASE}" | cut -d " " -f 1 | tr -d "\"" | sort -u >> /tmp/choices
  grep ".deb\"" "${DATABASE}" | cut -d " " -f 1 | tr -d "\"" | sort -u >> /tmp/choices
  grep ".tar.gz\"" "${DATABASE}" | cut -d " " -f 1 | tr -d "\"" | sort -u >> /tmp/choices
   

  #create /tmp/select for later lookup
  echo -n > /tmp/select
  COUNTER=0
  for LINE in $(cat /tmp/choices); do
    echo "${COUNTER}) ${LINE}" >> /tmp/select
    let COUNTER=COUNTER+1
  done
      
  #call menu_fnc
  menu_fc
  #evaluate response
  if [ "$(cat /tmp/reply)" = "0" ]; then	# handle cancel
	echo "Canceled - exit"
	cleanup
	exit
  else
	menuitem=$(cat /tmp/reply) 2> /dev/null
	PETSEL=$(cat /tmp/select | grep ^$menuitem\) | cut -d " " -f 2)
  fi
  echo "Please wait..."
fi

#lookup from db
grep "^\"${PETSEL}\" " "${DATABASE}" | cut -d "\"" -f 4 | sort -u > /tmp/"${PETSEL}".files 
sync

echo -n >/tmp/dirnames
#uninstall...
while read ONEFILE; do

  CORE=""
  REMFILE=""
  if [ -d "${ONEFILE}" ]; then
    echo "${ONEFILE}" >> /tmp/dirnames
  else
    #check if file is shared (check /root/.packages/allfiles.txt
    COUNT=`grep -c " \"${ONEFILE}\"$" "${DATABASE}"`
    #check if COUNT=0 - if filename contains special charachters
    [ -f "${ONEFILE}" ] || [ -h "${ONEFILE}" ] && [ "${COUNT}" = "0" ] && COUNT=1
    #optimize logic and reduce file-reads
    [ -f "${ONEFILE}" ] || [ -h "${ONEFILE}" ] && REMFILE="yes"
    [ -e /initrd/pup_ro2"${ONEFILE}" ] && CORE="yes"
    FILETYPE=""
    #file, not shared, core
    [ "${REMFILE}" = "yes" ] && [ "${COUNT}" = "1" ] && [ "${CORE}" = "yes" ] && FILETYPE=2 
    #file, shared, core
    [ "${REMFILE}" = "yes" ] && [ ! "${COUNT}" = "1" ] && [ "${CORE}" = "yes" ] && FILETYPE=3 
    #file, not shared, not core
    [ "${REMFILE}" = "yes" ] && [ "${COUNT}" = "1" ] && [ ! "${CORE}" = "yes" ] && FILETYPE=4 
     #file, shared, not core
    [ "${REMFILE}" = "yes" ] && [ ! "${COUNT}" = "1" ] && [ ! "${CORE}" = "yes" ] && FILETYPE=5
    
    case "$FILETYPE" in
      2)  #echo "Type 2" 
          cp -af /initrd/pup_ro2"${ONEFILE}" "${ONEFILE}";
          grep -v "^\"${PETSEL}\" \"${ONEFILE}\"$" "${DATABASE}" > "${DATABASE}".temp
          mv -f "${DATABASE}".temp "${DATABASE}";;
    3|5)  #echo "Type 3 or 5"
          grep -v "^\"${PETSEL}\" \"${ONEFILE}\"$" "${DATABASE}" > "${DATABASE}".temp
          mv -f "${DATABASE}".temp "${DATABASE}";;
      4)  #echo "Type 4 ${ONEFILE}";
          rm -f "${ONEFILE}";
          grep -v "^\"${PETSEL}\" \"${ONEFILE}\"$" "${DATABASE}" > "${DATABASE}".temp;
          mv -f "${DATABASE}".temp "${DATABASE}";;
      *)  echo "Unknown file-type for ${ONEFILE}";
          #this might be files with special symbols in, lets try to remove them
          rm -f "${ONEFILE}";;
    esac

    #catch dirs
    dirname "${ONEFILE}" >> /tmp/dirnames
  fi	#end not dir...

done </tmp/"${PETSEL}".files 

#check dirs
sort -u /tmp/dirnames > /tmp/dirnames.tmp
mv /tmp/dirnames.tmp /tmp/dirnames

while read ONEDIR; do
  del_empty_dir "${ONEDIR}"
  #delete in db
  if [ -e /initrd/pup_ro2"${ONEDIR}" ]; then
    grep -v "^\"${PETSEL}\" \"${ONEDIR}" "${DATABASE}" > "${DATABASE}".temp
    mv -f "${DATABASE}".temp "${DATABASE}"
  fi
done </tmp/dirnames

#run any postinstall scripts....
if [ -f /root/.packages/"${PETSEL}".remove ]; then
  echo "trying to run puninstaller..."
  /root/.packages/"${PETSEL}".remove
  rm -f /root/.packages/"${PETSEL}".remove
fi

#validation
#VALIDATION=`grep "^\"${PETSEL}\" " /root/.packages/allfiles.txt`
grep "^\"${PETSEL}\" " /root/.packages/allfiles.txt | cut -d "\"" -f 4 | sort -u  > /tmp/validation
while read ONEFILE; do
  [ -f "${ONEFILE}" ] && VALIDATION="${VALIDATION} ${ONEFILE}"
done </tmp/validation

if [ ! "${VALIDATION}" = "" ]; then
  TEXT="Package - ${PETSEL} - has been removed but the
following files or directories might not have been
removed:
${VALIDATION}
You might have to remove them manually.
"	
  message_fc	
else
  TEXT="Package - ${PETSEL} - has been removed.
"
  message_fc
fi
#clear database refs
grep -v "^\"${PETSEL}\" " /root/.packages/allfiles.txt > /root/.packages/allfiles.txt.temp
mv -f /root/.packages/allfiles.txt.temp /root/.packages/allfiles.txt

rm -f /root/.packages/"${PETSEL}".files
grep -v "^\"${PETSEL}" /root/.packages/alienpackages.txt > /root/.packages/alienpackages.txt.temp
mv -f /root/.packages/alienpackages.txt.temp /root/.packages/alienpackages.txt
sync

cleanup
exit